package LightController;

import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;

import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.Condition;

import TronAdapter.*;

public class OnOffLightController extends TronThread
    implements MouseListener, WindowListener
{
    private enum Loc { Off, On }
    private Loc location;

    boolean alive = false;

    long startTime;
    long aTime; //time of activation

    OnOffTestFixture testFixture = null;
    Dimmer dimmer = null;

    Lock lock = null;
    Condition cond = null;

    final int mutant, levelCount;
    final int Tsw=2000; //2 seconds
    public OnOffLightController(int mutant, int levelCount)
    {
	super("OnOffLightController");
	assert(levelCount>0);
	location = Loc.Off;
	this.mutant = mutant;
	this.levelCount = levelCount;
	testFixture = new OnOffTestFixture(9999, this);
	//Dimmer is only used to manage graphics easily
	dimmer = new Dimmer(aTime, levelCount, mutant);
	addLevelListener(testFixture);
	dimmer.start();
    }

    public void run()
    {

	try { synchronized(dimmer) { while (!dimmer.alive) dimmer.wait(); }}
	catch(InterruptedException e){ System.err.println(e); System.exit(1);}

	System.out.println("OnOff Init: "+location);

	lock = new TronLock();
	cond = lock.newCondition();
	lock.lock();

	synchronized (this) { alive = true; notifyAll(); }
	

	aTime = TronThread.getTimeMillis();
	while (alive) {
	    System.out.println("State: "+location);
	    switch (location) {
	    case Off:
		try { cond.await(); }
		catch(InterruptedException e) { alive = false; }
		//do  nothing
		break;
	    case On:  
		try { cond.await(); }
		catch(InterruptedException e) { alive = false; }
		//do  nothing
		break;
	    default:
		alive = false;
	    }
	}

	System.out.println("IFace interrupted in "+location);
	dimmer.interrupt();
	testFixture.interrupt();
	lock.unlock();

    }

    public void handleTouch()
    {
	synchronized (this) {
	    while (!alive) try {wait();}catch(InterruptedException e){}
	}
	lock.lock();
	startTime = getTimeMillis();
	System.out.println("LC: handleTouch at "+(startTime-aTime)+" ("+startTime+")");
	switch (location) {
	case Off:
	    location = Loc.On;
	    dimmer.setLevel(levelCount);
	    cond.signalAll();//wake up controller thread (see run method above)
	    break;
	case On:
	    location = Loc.Off;
	    dimmer.setLevel(0);
	    cond.signalAll();//wake up controller thread
	    break;
	default:
	    System.out.println("IFace: cannot accept handleTouch in "+
			       location);
	    //	    interrupt();
	}
	lock.unlock();
    }


    public void addLevelListener(LevelListener listener) {
	dimmer.addLevelListener(listener);
    }

    public void mouseClicked(MouseEvent e){
	if (!TronThread.simtime()) handleTouch();
	else {
	    System.err.println("##### Simulated time framework does not allow outside interraction. #####");
	    java.awt.Toolkit.getDefaultToolkit().beep();
	}
    }
    public void mouseEntered(MouseEvent e){}
    public void mouseExited(MouseEvent e){}
    public void mousePressed(MouseEvent e){}
    public void mouseReleased(MouseEvent e){}

    public void windowActivated(WindowEvent e) {}
    public void windowClosed(WindowEvent e) {}
    public void windowClosing(WindowEvent e) { System.exit(0); }
    public void windowDeactivated(WindowEvent e) {}
    public void windowDeiconified(WindowEvent e) {}
    public void windowIconified(WindowEvent e) {}
    public void windowOpened(WindowEvent e) {}

    public static void main(String args[]) {
	int mutant = 0;
	boolean noGUI = false;
	int i = 0;
	while (i<args.length) {
	    if ("-N".equals(args[i])){
		noGUI = true; i++;
	    } else if ("-M".equals(args[i])){
		if (i+1<args.length) mutant = Integer.parseInt(args[i+1]);
		else {
		    System.err.println("Specify mutant id.");
		    return ;
		}
		i += 2 ;
	    } else if ("-C".equals(args[i])) {
		if (i+2<args.length) {
		    int port = Integer.parseInt(args[i+2]);
		    if (port <= 0) {
			System.err.println("The specified port ("+args[i+2]+
					   ") is not in valid range.");
			return;
		    }
		    System.out.println("Setting simulated clock to "+
				       args[i+1]+":"+port);
		    TronThread.setRemoteClock(args[i+1], port);
		    i += 3;
		} else {
		    System.err.println("Specify simulated clock, like: "+
				       "-C localhost 6521");
		    return ;
		}
	    } else {
		System.err.println("Uninterpreted option: "+args[i]);
		i++;
	    }
	}
	if (mutant>=0 && mutant<=2) {
	    OnOffLightController lc = new OnOffLightController(mutant, 10);
	    System.out.println("LC constructed");
	    if (!noGUI) {
		Frame f = new Frame("OnOff LightController");
		f.addWindowListener(lc);
		LevelHistory lh = new LevelHistory(10, 0);
		lh.addMouseListener(lc);
		lc.addLevelListener(lh);
		LevelBar lb = new LevelBar(10, 0);
		lb.addMouseListener(lc);
		lc.addLevelListener(lb);
		LevelLamp lamp = new LevelLamp(10, 0);
		lamp.addMouseListener(lc);
		lc.addLevelListener(lamp);

		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		f.setLayout(gridbag);
		c.fill = GridBagConstraints.BOTH; c.weighty = 1.0;
		c.weightx = 9.0;
		gridbag.setConstraints(lh, c); f.add(lh);
		c.weightx = 1.0;
		gridbag.setConstraints(lb, c); f.add(lb);
		c.weightx = 5.0;
		c.gridwidth = GridBagConstraints.REMAINDER; //end row
		gridbag.setConstraints(lamp, c); f.add(lamp);
		f.setBounds(0, 0, 400, 202); f.setVisible(true);
		lh.waitForReady();
	    }
	    lc.start();
	    System.out.println("LC started");
	    try { lc.join(); }
	    catch (InterruptedException e) {}
	    System.out.println("LightController terminated");
	} else System.out.println("Unknown Mutant");
    }
}
