package TronAdapter;

import java.util.concurrent.locks.Condition;
import java.util.concurrent.TimeUnit;
import java.util.Date;

public class TronCondition implements Condition
{
    public final int id;
    public final TronLock lock;
    public TronCondition(TronLock l)
    {
	TronThread t = (TronThread)Thread.currentThread();
	id = t.condInit();
	lock = l;
    }

    public void await() throws InterruptedException
    {
	TronThread t = (TronThread)Thread.currentThread();
	if (lock.getOwner() != t)
	    throw new IllegalMonitorStateException("waiter is not lock owner");
	lock.setOwner(null);
	t.condWait(this);
	lock.setOwner(t);
    }

    public boolean await(long time, TimeUnit unit) throws InterruptedException
    {
	TronThread t = (TronThread)Thread.currentThread();
	if (lock.getOwner() != t)
	    throw new IllegalMonitorStateException("waiter is not lock owner");
	boolean res;
	lock.setOwner(null);
	res = t.condDelay(this, time, unit);
	lock.setOwner(t);
	return res;
    }

    public long awaitNanos(long nanosTimeout) throws InterruptedException
    {
	throw new UnsupportedOperationException("not implemented");
    }

    public void awaitUninterruptibly(){
	throw new UnsupportedOperationException("not implemented");
    }

    public boolean awaitUntil(Date deadline) throws InterruptedException
    {
	TronThread t = (TronThread)Thread.currentThread();
	if (lock.getOwner() != t)
	    throw new IllegalMonitorStateException("waiter is not lock owner");
	boolean res;
	lock.setOwner(null);
	res = t.condWait(this, deadline.getTime());
	lock.setOwner(t);
	return res;
    }

    public void signal()
    {
	TronThread t = (TronThread)Thread.currentThread();
	if (lock.getOwner() != t)
	    throw new IllegalMonitorStateException("thread is not lock owner");
	t.condSignal(this);
    }
    public void signalAll()
    {
	TronThread t = (TronThread)Thread.currentThread();
	if (lock.getOwner() != t)
	    throw new IllegalMonitorStateException("thread is not lock owner");
	t.condBroadcast(this);
    }
}
